@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: downloader-view.c 7351 2007-09-07 23:33:54Z cosimoc $
+ *  $Id: downloader-view.c 7964 2008-02-17 23:23:36Z xan $
  */
 
 #include "config.h" 
@@ -28,6 +28,8 @@
 #include "ephy-stock-icons.h"
 #include "ephy-gui.h"
 #include "ephy-debug.h"
+#include "ephy-prefs.h"
+#include "eel-gconf-extensions.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtkversion.h>
@@ -46,10 +48,12 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkstatusicon.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomeui/gnome-icon-lookup.h>
 
-#define CONF_DOWNLOADING_SHOW_DETAILS "/apps/epiphany/dialogs/downloader_show_details"
+
+#ifdef HAVE_LIBNOTIFY
+#include <libnotify/notify.h>
+#endif
 
 enum
 {
@@ -83,8 +87,13 @@ struct _DownloaderViewPrivate
 
 	GtkStatusIcon *status_icon;
 
+#ifdef HAVE_LIBNOTIFY
+	NotifyNotification *notification;
+#endif
+
 	guint idle_unref : 1;
 	guint source_id;
+	guint notification_timeout;
 };
 
 enum
@@ -129,46 +138,26 @@ download_dialog_delete_event_cb (GtkWidget *window,
 				 GdkEventAny *event,
 				 DownloaderView *dv);
 
-static GObjectClass *parent_class = NULL;
+#ifdef HAVE_LIBNOTIFY
+static void
+show_notification_window (DownloaderView *dv);
+#endif
 
-GType
-downloader_view_get_type (void)
-{
-       static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (DownloaderViewClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) downloader_view_class_init,
-			NULL, /* class_finalize */
-			NULL, /* class_data */
-			sizeof (DownloaderView),
-			0,    /* n_preallocs */
-			(GInstanceInitFunc) downloader_view_init
-		};
-
-		type = g_type_register_static (EPHY_TYPE_DIALOG,
-					       "DownloaderView",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (DownloaderView, downloader_view, EPHY_TYPE_DIALOG)
 
 static void
 downloader_view_class_init (DownloaderViewClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = downloader_view_finalize;
 
 	g_type_class_add_private (object_class, sizeof(DownloaderViewPrivate));
+
+#ifdef HAVE_LIBNOTIFY
+	notify_init (PACKAGE);
+#endif
+
 }
 
 static void
@@ -176,11 +165,13 @@ show_downloader_cb (DownloaderView *dv)
 {
 	if (!gtk_window_has_toplevel_focus (GTK_WINDOW (dv->priv->window)))
 	{
-		gtk_window_present (GTK_WINDOW (dv->priv->window));
+		ephy_dialog_show (EPHY_DIALOG (dv));
+		eel_gconf_set_boolean (CONF_DOWNLOADS_HIDDEN, FALSE);
 	}
 	else
 	{
-		gtk_widget_hide (dv->priv->window);
+		ephy_dialog_hide (EPHY_DIALOG (dv));
+		eel_gconf_set_boolean (CONF_DOWNLOADS_HIDDEN, TRUE);
 	}
 }
 
@@ -215,6 +206,10 @@ show_status_icon (DownloaderView *dv)
 
 	priv->status_icon = gtk_status_icon_new_from_stock (STOCK_DOWNLOAD);
 
+#ifdef HAVE_LIBNOTIFY
+	notify_notification_attach_to_status_icon (priv->notification, priv->status_icon);
+#endif
+
 	g_signal_connect_swapped (priv->status_icon, "activate",
 				  G_CALLBACK (show_downloader_cb), dv);
 	g_signal_connect (dv->priv->status_icon, "popup-menu",
@@ -296,10 +291,22 @@ downloader_view_finalize (GObject *object)
 		g_source_remove (priv->source_id);
 		priv->source_id = 0;
 	}
+	
+	if (priv->notification_timeout != 0)
+	{
+		g_source_remove (priv->notification_timeout);
+	}
 
 	g_hash_table_destroy (dv->priv->downloads_hash);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (downloader_view_parent_class)->finalize (object);
+
+#ifdef HAVE_LIBNOTIFY	
+	if (notify_is_initted ())
+	{
+		notify_uninit ();
+	}
+#endif
 
 	if (idle_unref)
 	{
@@ -354,21 +361,22 @@ update_buttons (DownloaderView *dv)
 	GtkTreeSelection *selection;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	GValue val = {0, };
-	EphyDownload *download;
 	EphyDownloadState state;
+	EphyDownload *download;
 	gboolean pause_enabled = FALSE;
 	gboolean abort_enabled = FALSE;
 	gboolean label_pause = TRUE;
+	GList *selected = NULL;
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(dv->priv->treeview));
+	selected = gtk_tree_selection_get_selected_rows (selection, &model);
 
-	if (gtk_tree_selection_get_selected (selection, &model, &iter))
+	if (g_list_length (selected) == 1)
 	{
-		gtk_tree_model_get_value (model, &iter, COL_DOWNLOAD_OBJECT, &val);
-		download = g_value_get_object (&val);
-		g_value_unset (&val);
-
+		if (gtk_tree_model_get_iter (model, &iter, selected->data) != TRUE)
+			return;
+		
+		gtk_tree_model_get (model, &iter, COL_DOWNLOAD_OBJECT, &download, -1);
 		state = ephy_download_get_state (download);
 
 		switch (state)
@@ -387,6 +395,14 @@ update_buttons (DownloaderView *dv)
 			break;
 		}
 	}
+	else
+	{
+		abort_enabled = TRUE;
+	}
+	
+	g_list_foreach (selected, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (selected);
+	
 	gtk_widget_set_sensitive (dv->priv->pause_button, pause_enabled);
 	gtk_widget_set_sensitive (dv->priv->abort_button, abort_enabled);
 	gtk_button_set_label (GTK_BUTTON (dv->priv->pause_button),
@@ -405,15 +421,41 @@ update_download_row (DownloaderView *dv, EphyDownload *download)
 	struct tm;
 	int percent = 0;
 
+#ifdef HAVE_LIBNOTIFY
+	char *downloaded;
+#endif
+
 	row_ref = get_row_from_download (dv, download);
 	g_return_if_fail (row_ref != NULL);
 
 	/* State special casing */
 	state = ephy_download_get_state (download);
+
+	total = ephy_download_get_total_progress (download);
+	current = ephy_download_get_current_progress (download);
+
+	cur_progress = g_format_size_for_display (current);
+
+	name = ephy_download_get_name (download);
+	
 	switch (state)
 	{
 	case EPHY_DOWNLOAD_COMPLETED:
 		downloader_view_remove_download (dv, download);
+		
+#ifdef HAVE_LIBNOTIFY
+		downloaded = g_strdup_printf (_("The file “%s” has been downloaded."), 
+						name);
+		notify_notification_update (dv->priv->notification,
+			_("Download finished"), 
+			downloaded, 
+			GTK_STOCK_INFO);
+			
+		show_notification_window (dv);
+		
+		g_free (downloaded);
+#endif
+
 		return;
 	case EPHY_DOWNLOAD_PAUSED:
 	case EPHY_DOWNLOAD_DOWNLOADING:
@@ -424,18 +466,11 @@ update_download_row (DownloaderView *dv, EphyDownload *download)
 		break;
 	}
 
-	total = ephy_download_get_total_progress (download);
-	current = ephy_download_get_current_progress (download);
-
-	cur_progress = gnome_vfs_format_file_size_for_display (current);
-
-	name = ephy_download_get_name (download);
-
 	if (total != -1 && current != -1)
 	{
 		char *total_progress;
 
-		total_progress = gnome_vfs_format_file_size_for_display (total);
+		total_progress = g_format_size_for_display (total);
 		/* translators: first %s is filename, "%s of %s" is current/total file size */
 		file = g_strdup_printf (_("%s\n%s of %s"), name,
 					cur_progress, total_progress);
@@ -508,6 +543,29 @@ update_buttons_timeout_cb (DownloaderView *dv)
 	return FALSE;
 }
 
+#ifdef HAVE_LIBNOTIFY
+static gboolean
+queue_show_notification (DownloaderView *dv)
+{
+	if (gtk_status_icon_is_embedded (dv->priv->status_icon))
+	{
+		notify_notification_show (dv->priv->notification, NULL);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static void
+show_notification_window (DownloaderView *dv)
+{
+	if (gtk_status_icon_is_embedded (dv->priv->status_icon))
+		notify_notification_show (dv->priv->notification, NULL);
+	else
+		dv->priv->notification_timeout = g_timeout_add_seconds (1, (GSourceFunc) queue_show_notification, dv);
+}
+#endif
+
 void
 downloader_view_add_download (DownloaderView *dv,
 			      EphyDownload *download)
@@ -521,6 +579,11 @@ downloader_view_add_download (DownloaderView *dv,
 	GdkPixbuf *pixbuf;
 	char *mime, *icon_name;
 	int width = 16, height = 16;
+	GValue visible = {0, };
+
+#ifdef HAVE_LIBNOTIFY
+	char *downloading;
+#endif
 
 	g_object_ref (download);
 
@@ -549,7 +612,33 @@ downloader_view_add_download (DownloaderView *dv,
 				 G_CALLBACK (download_changed_cb), dv, 0);
 
 	/* Show it already */
-	ephy_dialog_show (EPHY_DIALOG (dv));
+	g_value_init (&visible, G_TYPE_BOOLEAN);
+	g_object_get_property (G_OBJECT(dv->priv->window), "visible", &visible);
+	
+	if (eel_gconf_get_boolean (CONF_DOWNLOADS_HIDDEN) && !g_value_get_boolean (&visible))
+	{
+
+#ifdef HAVE_LIBNOTIFY
+		downloading = g_strdup_printf(_("The file “%s” has been added to the downloads queue."), 
+						ephy_download_get_name (download));
+		notify_notification_update (dv->priv->notification,
+					_("Download started"), 
+					downloading, 
+					GTK_STOCK_INFO);
+		
+		show_notification_window (dv);
+
+		g_free (downloading);
+#endif
+
+		ephy_dialog_hide (EPHY_DIALOG (dv));
+	}
+	else
+	{
+		ephy_dialog_show (EPHY_DIALOG (dv));
+	}
+	
+	g_value_unset (&visible);
 	
 	mime =  ephy_download_get_mime (download);
 
@@ -729,7 +818,15 @@ downloader_view_build_ui (DownloaderView *dv)
 	gtk_window_set_icon_name (GTK_WINDOW (priv->window), STOCK_DOWNLOAD);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 	g_signal_connect (selection, "changed", G_CALLBACK (selection_changed), dv);
+	
+#ifdef HAVE_LIBNOTIFY
+	priv->notification = notify_notification_new (" ", " ", GTK_STOCK_INFO, NULL);
+	notify_notification_set_timeout (priv->notification, NOTIFY_EXPIRES_DEFAULT);
+	notify_notification_set_urgency (priv->notification, NOTIFY_URGENCY_LOW);
+#endif
+
 }
 
 static void
@@ -834,25 +931,41 @@ downloader_view_remove_download (DownloaderView *dv, EphyDownload *download)
 static void
 download_dialog_stop (DownloaderView *dv)
 {
-	GValue val = {0, };
 	GtkTreeSelection *selection;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
-	gpointer download;
+	GList *selected = NULL;
+	GList *downloads = NULL;
+	GList *l = NULL;
+	EphyDownload *download;
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(dv->priv->treeview));
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW(dv->priv->treeview));
 	
-	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) return;
-
-	gtk_tree_model_get_value (model, &iter, COL_DOWNLOAD_OBJECT, &val);
+	selected = gtk_tree_selection_get_selected_rows (selection, &model);
 	
-	download = g_value_get_object (&val);
-	g_return_if_fail (download != NULL);
-
-	ephy_download_cancel ((EphyDownload*)download);
-	downloader_view_remove_download (dv, download);
-
-	g_value_unset (&val);
+	for (l = selected; l; l = l->next)
+	{
+		if (!gtk_tree_model_get_iter (model, &iter, l->data)) continue;
+		
+		gtk_tree_model_get (model, &iter, COL_DOWNLOAD_OBJECT, &download, -1);
+		downloads = g_list_append (downloads, download);
+	}
+	
+	/* We have to kill the downloads separately (not in the previous for) 
+	 * because otherwise selection would change.
+	 */
+	for (l = downloads; l; l = l->next)
+	{
+		if (!l->data) continue;
+		ephy_download_cancel ((EphyDownload*) l->data);
+		downloader_view_remove_download (dv, l->data);
+		g_object_unref (l->data);
+	}
+	
+	g_list_foreach (selected, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (selected);
+	g_list_free (downloads);
 }
 
 static void
