@@ -16,9 +16,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-favicon-cache.c,v 1.61 2006/12/17 03:06:12 bjornl Exp $
+ *  $Id: ephy-favicon-cache.c 7964 2008-02-17 23:23:36Z xan $
  */
 
 #include "config.h"
@@ -36,22 +36,23 @@
 #include "ephy-node-common.h"
 #include "ephy-node.h"
 #include "ephy-debug.h"
+#include "ephy-glib-compat.h"
 
 #include <glib/gstdio.h>
+#include <gio/gio.h>
 #include <libgnomeui/libgnomeui.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-directory.h>
 
 #define EPHY_FAVICON_CACHE_XML_ROOT    (const xmlChar *)"ephy_favicons_cache"
 #define EPHY_FAVICON_CACHE_XML_VERSION (const xmlChar *)"1.1"
 
 #define EPHY_FAVICON_CACHE_OBSOLETE_DAYS 30
+#define SECS_PER_DAY (60*60*24)
 
-/* how often to save the cache, in milliseconds */
-#define CACHE_SAVE_INTERVAL	10 * 60 * 1000 /* ms */
+/* how often to save the cache, in seconds */
+#define CACHE_SAVE_INTERVAL	(10 * 60) /* seconds */
 
-/* how often to delete old pixbufs from the cache, in milliseconds */
-#define CACHE_CLEANUP_INTERVAL	5 * 60 * 1000 /* ms */
+/* how often to delete old pixbufs from the cache, in seconds */
+#define CACHE_CLEANUP_INTERVAL	(5 * 60) /* seconds */
 
 /* how long to keep pixbufs in cache, in seconds. This should be longer than CACHE_CLEANUP_INTERVAL */
 #define PIXBUF_CACHE_KEEP_TIME	10 * 60 /* s */
@@ -89,7 +90,7 @@ typedef struct
 	guint load_failed : 1;
 } PixbufCacheEntry;
 
-typedef gboolean (* FilterFunc) (EphyNode*, GDate *);
+typedef gboolean (* FilterFunc) (EphyNode*, time_t);
 
 enum
 {
@@ -115,45 +116,15 @@ enum
 	NEEDS_MASK	 = 0x3f
 };
 
-static guint signals[LAST_SIGNAL] = { 0 };
+static guint signals[LAST_SIGNAL];
 
-static GObjectClass *parent_class = NULL;
-
-GType
-ephy_favicon_cache_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyFaviconCacheClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_favicon_cache_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyFaviconCache),
-			0,
-			(GInstanceInitFunc) ephy_favicon_cache_init
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "EphyFaviconCache",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (EphyFaviconCache, ephy_favicon_cache, G_TYPE_OBJECT)
 
 static void
 ephy_favicon_cache_class_init (EphyFaviconCacheClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = ephy_favicon_cache_finalize;
 
 	signals[CHANGED] =
@@ -188,23 +159,13 @@ pixbuf_cache_entry_free (PixbufCacheEntry *entry)
 }
 
 static gboolean
-icon_is_obsolete (EphyNode *node, GDate *now)
+icon_is_obsolete (EphyNode *node, time_t now)
 {
 	int last_visit;
-	GDate date;
 
 	last_visit = ephy_node_get_property_int
 		(node, EPHY_NODE_FAVICON_PROP_LAST_USED);
-
-	g_date_clear (&date, 1);
-#if GLIB_CHECK_VERSION (2,9,0)
-        g_date_set_time_t (&date, last_visit);
-#else
-	g_date_set_time (&date, last_visit);
-#endif
-
-	return (g_date_days_between (&date, now) >=
-		EPHY_FAVICON_CACHE_OBSOLETE_DAYS);
+	return now - last_visit >= EPHY_FAVICON_CACHE_OBSOLETE_DAYS*SECS_PER_DAY;
 }
 
 static void
@@ -238,14 +199,9 @@ remove_obsolete_icons (EphyFaviconCache *cache,
 	EphyFaviconCachePrivate *priv = cache->priv;
 	GPtrArray *children;
 	int i;
-	GDate current_date;
+	time_t now;
 
-	g_date_clear (&current_date, 1);
-#if GLIB_CHECK_VERSION (2,9,0)
-        g_date_set_time_t (&current_date, time (NULL));
-#else
-	g_date_set_time (&current_date, time (NULL));
-#endif
+	now = time (NULL);
 
 	children = ephy_node_get_children (priv->icons);
 	for (i = (int) children->len - 1; i >= 0; i--)
@@ -254,7 +210,7 @@ remove_obsolete_icons (EphyFaviconCache *cache,
 
 		kid = g_ptr_array_index (children, i);
 
-		if (!filter || filter (kid, &current_date))
+		if (!filter || filter (kid, now))
 		{
 			const char *filename;
 			char *path;
@@ -263,7 +219,10 @@ remove_obsolete_icons (EphyFaviconCache *cache,
 				(kid, EPHY_NODE_FAVICON_PROP_FILENAME);
 			path = g_build_filename (priv->directory,
 						 filename, NULL);
-			gnome_vfs_unlink (path);
+			if (g_unlink (path) < 0)
+			{
+				g_warning ("Unable to delete %s", path);
+			}
 
 			g_free (path);
 			ephy_node_unref (kid);
@@ -395,10 +354,10 @@ ephy_favicon_cache_init (EphyFaviconCache *cache)
 	g_signal_connect_object (embed_shell, "prepare-close",
 				 G_CALLBACK (prepare_close_cb), cache, 0);
 
-	priv->autosave_timeout = g_timeout_add (CACHE_SAVE_INTERVAL,
+	priv->autosave_timeout = g_timeout_add_seconds (CACHE_SAVE_INTERVAL,
 						(GSourceFunc) periodic_save_cb,
 						cache);
-	priv->cleanup_timeout = g_timeout_add (CACHE_CLEANUP_INTERVAL,
+	priv->cleanup_timeout = g_timeout_add_seconds (CACHE_CLEANUP_INTERVAL,
 					       (GSourceFunc) periodic_cleanup_cb,
 					       cache);
 }
@@ -425,29 +384,28 @@ kill_download (const char *key,
 	return TRUE;
 }
 
-static gboolean
-delete_file (const char *rel_path,
-	     GnomeVFSFileInfo *info,
-	     gboolean rec_will_loop,
-	     EphyFaviconCache *cache,
-	     gboolean *recurse)
+static void
+delete_file (GFile *dir,
+	     GFileInfo *file_info)
 {
-	EphyFaviconCachePrivate *priv = cache->priv;
-	char *path;
-
-	*recurse = FALSE;
-
-	g_return_val_if_fail (info != NULL, TRUE);
-
-	if ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_TYPE) == 0 ||
-	    info->type != GNOME_VFS_FILE_TYPE_REGULAR) return TRUE;
-
-	path = g_build_filename (priv->directory, rel_path, NULL);
-	gnome_vfs_unlink (path);
-	g_free (path);
-
-	/* continue with the visit */
-	return TRUE;
+	GFileType type;
+	
+	type = g_file_info_get_file_type (file_info);
+	
+	if (type == G_FILE_TYPE_REGULAR)
+	{
+		char *path;
+		
+		path = g_build_filename (g_file_get_path (dir), 
+					 g_file_info_get_name (file_info),
+					 NULL);
+		if (g_unlink (path) < 0)
+		{
+			g_warning ("Unable to delete %s", path);
+		}
+		
+		g_free (path);
+	}							      
 }
 
 static void
@@ -485,7 +443,7 @@ ephy_favicon_cache_finalize (GObject *object)
 	LOG ("Requests: cached %" G_GUINT64_FORMAT " / total %" G_GUINT64_FORMAT " = %.3f",
 	     priv->cached, priv->requests, ((double) priv->cached) / ((double) priv->requests));
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (ephy_favicon_cache_parent_class)->finalize (object);
 }
 
 static void
@@ -522,8 +480,11 @@ favicon_download_cancelled_cb (EphyEmbedPersist *persist,
 	g_hash_table_remove (cache->priv->downloads_hash, url);
 
 	/* remove a partially downloaded file */
-	dest = ephy_embed_persist_get_dest (persist);
-	gnome_vfs_unlink (dest);
+		dest = ephy_embed_persist_get_dest (persist);
+	if (g_unlink (dest) < 0)
+	{
+		g_warning ("Unable to delete %s", dest);
+	}
 
 	/* FIXME: re-schedule to try again after n days? */
 
@@ -713,29 +674,35 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 					    (int) checklevel);
 	}
 
-	/* Now check the type. We renamed the file above, so gnome-vfs does NOT
+	/* Now check the type. We renamed the file above, so glib does NOT
 	 * fall back to extension checking if the slow mime check fails for 
 	 * whatever reason
 	 */
 	if (checklevel & NEEDS_TYPE_CHECK)
 	{
-		GnomeVFSFileInfo *info;
-		gboolean valid = FALSE, is_ao = FALSE;;
-
+		GFile *file;
+		GFileInfo *file_info;
+		const char *mime_type;
+		gboolean valid = FALSE, is_ao = FALSE;
+		
+		file = g_file_new_for_path (pix_file);
+	
 		/* Sniff mime type and check if it's safe to open */
-		info = gnome_vfs_file_info_new ();
-		if (gnome_vfs_get_file_info (pix_file, info,
-					     GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
-					     GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE) == GNOME_VFS_OK &&
-		    (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE) &&
-		    info->mime_type != NULL)
+		file_info = g_file_query_info (file,
+					       G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+					       0, NULL, NULL);
+		mime_type = g_file_info_get_content_type (file_info);
+		if (file_info == NULL)
 		{
-			valid = strcmp (info->mime_type, "image/x-ico") == 0 ||
-				strcmp (info->mime_type, "image/png") == 0 ||
-				strcmp (info->mime_type, "image/gif") == 0;
-			is_ao = strcmp (info->mime_type, "application/octet-stream") == 0;
+			return NULL;
 		}
-		gnome_vfs_file_info_unref (info);
+		valid = strcmp (mime_type, "image/x-ico") == 0 ||
+			strcmp (mime_type, "image/png") == 0 ||
+			strcmp (mime_type, "image/gif") == 0;
+		is_ao = strcmp (mime_type, "application/octet-stream") == 0;
+
+		g_object_unref (file_info);
+		g_object_unref (file);
 
 		/* As a special measure, we try to load an application/octet-stream file
 		 * as an ICO file, since we cannot detect a ICO file without .ico extension
@@ -848,6 +815,9 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 void
 ephy_favicon_cache_clear (EphyFaviconCache *cache)
 {
+	GFileEnumerator *file_enum;
+	GFile *dir;
+	GFileInfo *file_info = NULL;
 	EphyFaviconCachePrivate *priv = cache->priv;
 
 	g_return_if_fail (EPHY_IS_FAVICON_CACHE (cache));
@@ -856,10 +826,19 @@ ephy_favicon_cache_clear (EphyFaviconCache *cache)
 	ephy_favicon_cache_save (cache);
 
 	/* Now remove any remaining files from the cache directory */
-	gnome_vfs_directory_visit (priv->directory,
-				   GNOME_VFS_FILE_INFO_DEFAULT,
-				   GNOME_VFS_DIRECTORY_VISIT_SAMEFS |
-				   GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK,
-				   (GnomeVFSDirectoryVisitFunc) delete_file,
-				   cache);
+	dir = g_file_new_for_path (priv->directory);
+	file_enum = g_file_enumerate_children (dir,
+					       G_FILE_ATTRIBUTE_STANDARD_TYPE ","
+					       G_FILE_ATTRIBUTE_STANDARD_NAME ","
+					       G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+					       0, NULL, NULL);
+	file_info = g_file_enumerator_next_file (file_enum, NULL, NULL);
+	while (file_info != NULL)
+	{
+		delete_file (dir, file_info);
+		file_info = g_file_enumerator_next_file (file_enum, NULL, NULL);
+		g_object_unref (file_info);
+	}
+	g_object_unref (dir);
+	g_file_enumerator_close (file_enum, NULL, NULL);
 }
