@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: EphyBrowser.h 7626 2007-11-05 19:54:41Z xan $
+ *  $Id: EphyBrowser.h 7896 2008-01-28 20:37:51Z cosimoc $
  */
 
 #ifndef EPHY_BROWSER_H
@@ -187,7 +187,7 @@ public:
 
 	nsresult LoadURI(const char *aURI, 
 			 PRUint32 aLoadFlags = nsIWebNavigation::LOAD_FLAGS_NONE, 
-			 nsIURI *aURI = nsnull);
+			 nsIURI *aReferrer = nsnull);
 
         EphyEmbedDocumentType GetDocumentType ();
 
