@@ -14,9 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: FilePicker.cpp 6588 2006-09-13 11:34:25Z chpe $
+ *  $Id: FilePicker.cpp 7967 2008-02-18 13:40:59Z chpe $
  */
 
 #include "mozilla-config.h"
@@ -38,13 +38,15 @@
 #include <nsIServiceManager.h>
 #include <nsIURI.h>
 #include <nsNetCID.h>
+#include <nsComponentManagerUtils.h>
+#include <nsServiceManagerUtils.h>
+#include <nsXPCOMCID.h>
 
 #include "ephy-debug.h"
 #include "ephy-gui.h"
 #include "ephy-prefs.h"
 
-#include "AutoJSContextStack.h"
-#include "AutoWindowModalState.h"
+#include "AutoModalDialog.h"
 #include "EphyUtils.h"
 
 #include "FilePicker.h"
@@ -70,27 +72,20 @@ GFilePicker::~GFilePicker()
 }
 
 /* void init (in nsIDOMWindow parent, in AString title, in short mode); */
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
 NS_IMETHODIMP GFilePicker::Init(nsIDOMWindow *parent, const nsAString& title, PRInt16 mode)
-#else
-NS_IMETHODIMP GFilePicker::Init(nsIDOMWindowInternal *parent, const PRUnichar *title, PRInt16 mode)
-#endif
 {
 	LOG ("GFilePicker::Init");
 
 	mParent = do_QueryInterface (parent);
 
 	GtkWidget *gtkparent = EphyUtils::FindGtkParent (parent);
-#if defined(MOZ_NSIFILEPICKER_NSASTRING_)
+
 	NS_ENSURE_TRUE (gtkparent, NS_ERROR_FAILURE);
-#endif
+
 
 	nsCString cTitle;
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
+
 	NS_UTF16ToCString (title, NS_CSTRING_ENCODING_UTF8, cTitle);
-#else
-	NS_UTF16ToCString (nsString(title), NS_CSTRING_ENCODING_UTF8, cTitle);
-#endif
 
 	mMode = mode;
 
@@ -187,28 +182,17 @@ NS_IMETHODIMP GFilePicker::AppendFilters(PRInt32 filterMask)
 }
 
 /* void appendFilter (in AString title, in AString filter); */
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
+
 NS_IMETHODIMP GFilePicker::AppendFilter(const nsAString& title, const nsAString& filter)
-#else
-NS_IMETHODIMP GFilePicker::AppendFilter(const PRUnichar *title, const PRUnichar *filter)
-#endif
 {
 	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
 	LOG ("GFilePicker::AppendFilter");
 
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
 	if (!filter.Length()) return NS_ERROR_FAILURE;
-#else
-	if (!filter) return NS_ERROR_FAILURE;
-#endif
 
 	nsCString pattern;
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
 	NS_UTF16ToCString (filter, NS_CSTRING_ENCODING_UTF8, pattern);
-#else
-	NS_UTF16ToCString (nsString(filter), NS_CSTRING_ENCODING_UTF8, pattern);
-#endif
 
 	char **patterns;
 	patterns = g_strsplit (pattern.get(), ";", -1);
@@ -223,11 +207,7 @@ NS_IMETHODIMP GFilePicker::AppendFilter(const PRUnichar *title, const PRUnichar
 	}
 
 	nsCString cTitle;
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
 	NS_UTF16ToCString (title, NS_CSTRING_ENCODING_UTF8, cTitle);
-#else
-	NS_UTF16ToCString (nsString(title), NS_CSTRING_ENCODING_UTF8, cTitle);
-#endif
 
 	gtk_file_filter_set_name (filth, cTitle.get());
 
@@ -239,30 +219,18 @@ NS_IMETHODIMP GFilePicker::AppendFilter(const PRUnichar *title, const PRUnichar
 }
 
 /* attribute AString defaultString; */
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
 NS_IMETHODIMP GFilePicker::GetDefaultString(nsAString& aDefaultString)
-#else
-NS_IMETHODIMP GFilePicker::GetDefaultString(PRUnichar **aDefaultString)
-#endif
 {
 	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
 	LOG ("GFilePicker::GetDefaultString");
 
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
 	aDefaultString = mDefaultString;
-#else
-	*aDefaultString = NS_StringCloneData (mDefaultString);
-#endif
 
 	return NS_OK;
 }
 
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
 NS_IMETHODIMP GFilePicker::SetDefaultString(const nsAString& aDefaultString)
-#else
-NS_IMETHODIMP GFilePicker::SetDefaultString(const PRUnichar *aDefaultString)
-#endif
 {
 	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
@@ -287,22 +255,14 @@ NS_IMETHODIMP GFilePicker::SetDefaultString(const PRUnichar *aDefaultString)
 }
 
 /* attribute AString defaultExtension; */
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
 NS_IMETHODIMP GFilePicker::GetDefaultExtension(nsAString& aDefaultExtension)
-#else
-NS_IMETHODIMP GFilePicker::GetDefaultExtension(PRUnichar **aDefaultExtension)
-#endif
 {
 	LOG ("GFilePicker::GetDefaultExtension");
 
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-#ifdef MOZ_NSIFILEPICKER_NSASTRING_
 NS_IMETHODIMP GFilePicker::SetDefaultExtension(const nsAString& aDefaultExtension)
-#else
-NS_IMETHODIMP GFilePicker::SetDefaultExtension(const PRUnichar *aDefaultExtension)
-#endif
 {
 	LOG ("GFilePicker::SetDefaultExtension");
 
@@ -429,12 +389,12 @@ NS_IMETHODIMP GFilePicker::GetFiles(nsISimpleEnumerator * *aFiles)
 /* short show (); */
 NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 {
-	nsresult rv;
-	AutoJSContextStack stack;
-	rv = stack.Init ();
-	if (NS_FAILED (rv)) return rv;
-
-	AutoWindowModalState (mParent);
+        AutoModalDialog modalDialog (mParent, PR_FALSE);
+        if (!modalDialog.ShouldShow ()) {
+          *_retval = nsIFilePicker::returnCancel;
+          return NS_OK;
+        }
+          
 	mParent = nsnull;
 
 	LOG ("GFilePicker::Show");
@@ -464,7 +424,7 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 
 	do
 	{
-		response = gtk_dialog_run (GTK_DIALOG (mDialog));
+		response = modalDialog.Run (GTK_DIALOG (mDialog));
 
 		g_free (filename);
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (mDialog));
