@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-file-helpers.c 7858 2008-01-13 20:42:01Z cosimoc $
+ *  $Id: ephy-file-helpers.c 7888 2008-01-24 19:14:16Z cosimoc $
  */
 
 #include "config.h"
@@ -603,7 +603,7 @@ ephy_file_launch_application (GAppInfo *app,
 	GAppLaunchContext *context;
 	GdkDisplay *display;
 	GdkScreen *screen;
-	
+
 	context = G_APP_LAUNCH_CONTEXT (eel_app_launch_context_new ());
 	if (widget)
 	{
@@ -702,14 +702,13 @@ ephy_file_launch_handler (const char *mime_type,
 }
 
 gboolean
-ephy_file_browse_to (const char *parameter,
+ephy_file_browse_to (GFile *file,
 		     guint32 user_time)
 {
-	GFile *file, *parent, *desktop;
+	GFile *parent, *desktop;
 	char *desktop_dir;
 	gboolean ret;
 
-	file = g_file_new_for_path (parameter);
 	desktop_dir = ephy_file_desktop_dir ();
 	desktop = g_file_new_for_path (desktop_dir);
 	
@@ -728,7 +727,6 @@ ephy_file_browse_to (const char *parameter,
 	}
 	
 	g_free (desktop_dir);
-	g_object_unref (file);
 
 	return ret;
 }
