@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-location-entry.c 7283 2007-08-19 07:43:17Z diegoe $
+ *  $Id: ephy-location-entry.c 7945 2008-02-15 01:10:36Z cosimoc $
  */
 
 #include "config.h"
@@ -38,6 +38,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtktoolbar.h>
 #include <gtk/gtkentry.h>
+#include <gtk/gtkentrycompletion.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcellrendererpixbuf.h>
@@ -69,7 +70,10 @@ struct _EphyLocationEntryPrivate
 	GdkColor secure_bg_colour;
 	GdkColor secure_fg_colour;
 
+	GtkCellRenderer *extracell;
+
 	char *before_completion;
+	char *saved_text;
 
 	guint text_col;
 	guint action_col;
@@ -82,30 +86,14 @@ struct _EphyLocationEntryPrivate
 	guint hash;
 
 	guint user_changed : 1;
+	guint can_redo : 1;
+	guint block_update : 1;
 	guint original_address : 1;
 	guint secure : 1;
 	guint apply_colours : 1;
 	guint needs_reset : 1;
 };
 
-static const struct
-{
-	const char *prefix;
-	int len;
-}
-web_prefixes [] =
-{
-	{ "http://www.", 11 },
-	{ "http://ftp.", 11 },
-	{ "http://", 7 },
-	{ "https://www.", 12 },
-	{ "https://", 8 },
-	{ "ftp://", 6},
-	{ "ftp://ftp.", 10},
-	{ "www.", 4 },
-	{ "ftp.", 4}
-};
-
 static const GtkTargetEntry url_drag_types [] =
 {
 	{ EPHY_DND_URL_TYPE,        0, 0 },
@@ -120,6 +108,12 @@ static void ephy_location_entry_class_init (EphyLocationEntryClass *klass);
 static void ephy_location_entry_init (EphyLocationEntry *le);
 static gboolean ephy_location_entry_reset_internal (EphyLocationEntry *, gboolean);
 
+static void extracell_data_func (GtkCellLayout *cell_layout,
+				GtkCellRenderer *cell,
+				GtkTreeModel *tree_model,
+				GtkTreeIter *iter,
+				gpointer data);
+
 static GObjectClass *parent_class = NULL;
 
 enum signalsEnum
@@ -172,6 +166,7 @@ ephy_location_entry_style_set (GtkWidget *widget,
 	EphyLocationEntryPrivate *priv = entry->priv;
 	GtkSettings *settings;
 	GdkColor *bg_colour = NULL, *fg_colour = NULL;
+	GdkColor title_fg_colour;
 	char *theme;
 	gboolean is_a11y_theme;
 
@@ -180,6 +175,12 @@ ephy_location_entry_style_set (GtkWidget *widget,
 		GTK_WIDGET_CLASS (parent_class)->style_set (widget, previous_style);
 	}
 
+	title_fg_colour = widget->style->text[GTK_STATE_INSENSITIVE];
+	g_object_set (entry->priv->extracell, 
+			"foreground-gdk", &title_fg_colour,
+			"foreground-set", TRUE,
+			NULL);
+
 	settings = gtk_settings_get_for_screen (gtk_widget_get_screen (widget));
 	g_object_get (settings, "gtk-theme-name", &theme, NULL);
 	is_a11y_theme = strstr (theme, "HighContrast") != NULL ||
@@ -223,6 +224,8 @@ ephy_location_entry_finalize (GObject *object)
 {
 	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (object);
 	EphyLocationEntryPrivate *priv = entry->priv;
+	
+	g_free (priv->saved_text);
 
 	if (priv->favicon != NULL)
 	{
@@ -247,7 +250,7 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 	signals[USER_CHANGED] = g_signal_new (
 		"user_changed", G_OBJECT_CLASS_TYPE (klass),
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET (EphyLocationEntryClass, user_changed),
+		G_STRUCT_OFFSET (EphyLocationEntryClass, user_changed),
 		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE,
@@ -267,7 +270,7 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 	signals[GET_LOCATION] = g_signal_new (
 		"get-location", G_OBJECT_CLASS_TYPE (klass),
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET (EphyLocationEntryClass, get_location),
+		G_STRUCT_OFFSET (EphyLocationEntryClass, get_location),
 		ephy_signal_accumulator_string, NULL,
 		ephy_marshal_STRING__VOID,
 		G_TYPE_STRING,
@@ -277,26 +280,26 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 	signals[GET_TITLE] = g_signal_new (
 		"get-title", G_OBJECT_CLASS_TYPE (klass),
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET (EphyLocationEntryClass, get_title),
+		G_STRUCT_OFFSET (EphyLocationEntryClass, get_title),
 		ephy_signal_accumulator_string, NULL,
 		ephy_marshal_STRING__VOID,
 		G_TYPE_STRING,
 		0,
 		G_TYPE_NONE);
 
-        gtk_widget_class_install_style_property (widget_class,
-                                                 g_param_spec_boxed ("secure-bg-color",
-                                                                     "Secure background colour",
-                                                                     "Background colour to use for secure sites",
-                                                                     GDK_TYPE_COLOR,
-                                                                     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+	gtk_widget_class_install_style_property (widget_class,
+						 g_param_spec_boxed ("secure-bg-color",
+								     "Secure background colour",
+								     "Background colour to use for secure sites",
+								     GDK_TYPE_COLOR,
+								     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
-        gtk_widget_class_install_style_property (widget_class,
-                                                 g_param_spec_boxed ("secure-fg-color",
-                                                                     "Secure foreground Colour",
-                                                                     "Foreground colour to use for secure sites",
-                                                                     GDK_TYPE_COLOR,
-                                                                     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+	gtk_widget_class_install_style_property (widget_class,
+						 g_param_spec_boxed ("secure-fg-color",
+								     "Secure foreground Colour",
+								     "Foreground colour to use for secure sites",
+								     GDK_TYPE_COLOR,
+								     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private (object_class, sizeof (EphyLocationEntryPrivate));
 }
@@ -345,7 +348,13 @@ editable_changed_cb (GtkEditable *editable,
 
 	update_address_state (entry);
 
-	if (priv->user_changed == FALSE) return;
+	if (priv->block_update == TRUE) 
+		return;
+	else
+	{
+		priv->user_changed = TRUE;
+		priv->can_redo = FALSE;
+	}	
 
 	update_favicon (entry);
 
@@ -402,6 +411,21 @@ entry_key_press_after_cb (GtkEntry *entry,
 
 		return TRUE;
 	}
+	
+	if ((event->keyval == GDK_Down || event->keyval == GDK_KP_Down)
+	    && state == 0)
+	{
+		/* If we are focusing the entry, with the cursor at the end of it
+		 * we emit the changed signal, so that the completion popup appears */
+		const char *string;
+		
+		string = gtk_entry_get_text (entry);
+		if (gtk_editable_get_position (GTK_EDITABLE (entry)) == strlen (string))
+		{
+			g_signal_emit_by_name (entry, "changed", 0);
+			return TRUE;
+		}
+	}
 
 	return FALSE;
 }
@@ -411,6 +435,8 @@ entry_activate_after_cb (GtkEntry *entry,
 			 EphyLocationEntry *lentry)
 {
 	EphyLocationEntryPrivate *priv = lentry->priv;
+	
+	priv->user_changed = FALSE;
 
 	if (priv->needs_reset)
 	{
@@ -419,88 +445,6 @@ entry_activate_after_cb (GtkEntry *entry,
 	}
 }
 
-static gboolean
-keyword_match (const char *list,
-	       const char *keyword)
-{
-	const char *p;
-	gsize keyword_len;
-
-	p = list;
-	keyword_len = strlen (keyword);
-
-	while (*p)
-	{
-		int i;
-
-		for (i = 0; i < keyword_len; i++)
-		{
-			if (p[i] != keyword[i])
-			{
-				goto next_token;
-			}
-		}
-	  
-		return TRUE;
-	  
-		next_token:
-
-		while (*p && !g_ascii_ispunct(*p) && !g_ascii_isspace(*p)) p++;
-		while (*p && (g_ascii_ispunct(*p) || g_ascii_isspace(*p))) p++;
-	}
-
-	return FALSE;
-}
-
-static gboolean
-completion_func (GtkEntryCompletion *completion,
-                 const char *key,
-		 GtkTreeIter *iter,
-		 gpointer data)
-{
-	int i, len_key, len_prefix;
-	char *item = NULL;
-	char *keywords = NULL;
-	gboolean ret = FALSE;
-	EphyLocationEntry *le = EPHY_LOCATION_ENTRY (data);
-	GtkTreeModel *model;
-
-	model = gtk_entry_completion_get_model (completion);
-
-	gtk_tree_model_get (model, iter,
-			    le->priv->text_col, &item,
-			    le->priv->keywords_col, &keywords,
-			    -1);
-
-	len_key = strlen (key);
-	if (!strncasecmp (key, item, len_key))
-	{
-		ret = TRUE;
-	}
-	else if (keyword_match (keywords, key))
-	{
-		ret = TRUE;
-	}
-	else
-	{
-		for (i = 0; i < G_N_ELEMENTS (web_prefixes); i++)
-		{
-			len_prefix = web_prefixes[i].len;
-			if (!strncmp (web_prefixes[i].prefix, item, len_prefix) &&
-			    !strncasecmp (key, item + len_prefix, len_key))
-			{
-				ret = TRUE;
-				break;
-			}
-		}
-	}
-
-	g_free (item);
-	g_free (keywords);
-
-	return ret;
-}
-
 static gboolean
 match_selected_cb (GtkEntryCompletion *completion,
 		   GtkTreeModel *model,
@@ -597,11 +541,26 @@ entry_clear_activate_cb (GtkMenuItem *item,
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 
-	priv->user_changed = FALSE;
+	priv->block_update = TRUE;
 	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), "");
+	priv->block_update = FALSE;
 	priv->user_changed = TRUE;
 }
 
+static void
+entry_redo_activate_cb (GtkMenuItem *item,
+			EphyLocationEntry *entry)
+{
+	ephy_location_entry_undo_reset (entry);
+}
+
+static void
+entry_undo_activate_cb (GtkMenuItem *item,
+			EphyLocationEntry *entry)
+{
+	ephy_location_entry_reset_internal (entry, FALSE);
+}
+
 static void
 entry_populate_popup_cb (GtkEntry *entry,
 			 GtkMenu *menu,
@@ -609,7 +568,7 @@ entry_populate_popup_cb (GtkEntry *entry,
 {
 	EphyLocationEntryPrivate *priv = lentry->priv;
 	GtkWidget *image;
-	GtkWidget *menuitem;
+	GtkWidget *clear_menuitem, *undo_menuitem, *redo_menuitem, *separator;
 	GList *children, *item;
 	int pos = 0, sep = 0;
 	gboolean is_editable;
@@ -622,13 +581,13 @@ entry_populate_popup_cb (GtkEntry *entry,
 	 * standard items in the GtkEntry context menu (Cut, Copy, Paste, Delete,
 	 * Select All, Input Methods and Insert Unicode control character.)
 	 */
-	menuitem = gtk_image_menu_item_new_with_mnemonic (_("Cl_ear"));
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(menuitem), image);
-	g_signal_connect (menuitem , "activate",
+	clear_menuitem = gtk_image_menu_item_new_with_mnemonic (_("Cl_ear"));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (clear_menuitem), image);
+	g_signal_connect (clear_menuitem , "activate",
 			  G_CALLBACK (entry_clear_activate_cb), lentry);
 	is_editable = gtk_editable_get_editable (GTK_EDITABLE (priv->icon_entry->entry));
-	gtk_widget_set_sensitive (menuitem, is_editable);
-	gtk_widget_show (menuitem);
+	gtk_widget_set_sensitive (clear_menuitem, is_editable);
+	gtk_widget_show (clear_menuitem);
 
 	/* search for the 2nd separator (the one after Select All) in the context
 	 * menu, and insert this menu item before it.
@@ -640,7 +599,25 @@ entry_populate_popup_cb (GtkEntry *entry,
 		if (GTK_IS_SEPARATOR_MENU_ITEM (item->data)) sep++;
 	}
 
-	gtk_menu_shell_insert (GTK_MENU_SHELL (menu), menuitem, pos - 1);
+	gtk_menu_shell_insert (GTK_MENU_SHELL (menu), clear_menuitem, pos - 1);
+	
+	undo_menuitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_UNDO, NULL);
+	gtk_widget_set_sensitive (undo_menuitem, priv->user_changed);
+	g_signal_connect (undo_menuitem, "activate",
+			  G_CALLBACK (entry_undo_activate_cb), lentry);
+	gtk_widget_show (undo_menuitem);
+	gtk_menu_shell_insert (GTK_MENU_SHELL (menu), undo_menuitem, 0);
+	
+	redo_menuitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_REDO, NULL);
+	gtk_widget_set_sensitive (redo_menuitem, priv->can_redo);
+	g_signal_connect (redo_menuitem, "activate",
+			  G_CALLBACK (entry_redo_activate_cb), lentry);
+	gtk_widget_show (redo_menuitem);
+	gtk_menu_shell_insert (GTK_MENU_SHELL (menu), redo_menuitem, 1);
+	
+	separator = gtk_separator_menu_item_new ();
+	gtk_widget_show (separator);
+	gtk_menu_shell_insert (GTK_MENU_SHELL (menu), separator, 2);
 }
 
 static void
@@ -833,8 +810,8 @@ icon_button_press_event_cb (GtkWidget *ebox,
 		EphyLocationEntryPrivate *priv = entry->priv;
 		
 		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (entry));
-        	gtk_window_set_focus (GTK_WINDOW(toplevel),
-                              	      priv->icon_entry->entry);
+		gtk_window_set_focus (GTK_WINDOW(toplevel),
+			      	      priv->icon_entry->entry);
 
 		gtk_editable_select_region (GTK_EDITABLE (priv->icon_entry->entry), 
 					    0, -1);
@@ -889,7 +866,7 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 			  G_CALLBACK (icon_button_press_event_cb), entry);
 
 	gtk_widget_set_tooltip_text (priv->icon_ebox,
-			             _("Drag and drop this icon to create a link to this page"));
+				     _("Drag and drop this icon to create a link to this page"));
 
 	priv->icon = gtk_image_new ();
 	gtk_container_add (GTK_CONTAINER (priv->icon_ebox), priv->icon);
@@ -945,7 +922,9 @@ ephy_location_entry_init (EphyLocationEntry *le)
 	p = EPHY_LOCATION_ENTRY_GET_PRIVATE (le);
 	le->priv = p;
 
-	p->user_changed = TRUE;
+	p->user_changed = FALSE;
+	p->block_update = FALSE;
+	p->saved_text = NULL;
 
 	ephy_location_entry_construct_contents (le);
 
@@ -990,13 +969,51 @@ cursor_on_match_cb  (GtkEntryCompletion *completion,
 	
 	entry = gtk_entry_completion_get_entry (completion);
 	gtk_entry_set_text (GTK_ENTRY (entry), item);
-        gtk_editable_set_position (GTK_EDITABLE (entry), -1);
+	gtk_editable_set_position (GTK_EDITABLE (entry), -1);
 
 	g_free (item);
 
 	return TRUE;
 }
 
+static void
+extracell_data_func (GtkCellLayout *cell_layout,
+			GtkCellRenderer *cell,
+			GtkTreeModel *tree_model,
+			GtkTreeIter *iter,
+			gpointer data)
+{
+	char *cdata;
+	GValue visible = { 0, };
+	GValue text = { 0, };
+	
+	gtk_tree_model_get (tree_model, iter, GPOINTER_TO_UINT(data), &cdata, -1);
+
+	g_value_init (&text, G_TYPE_STRING);
+	g_value_init (&visible, G_TYPE_BOOLEAN);
+	
+	g_value_take_string (&text, cdata);
+	g_value_set_boolean (&visible, (cdata != NULL));
+
+	g_object_set_property (G_OBJECT (cell), "text", &text);
+	g_object_set_property (G_OBJECT (cell), "visible", &visible);
+	
+	g_value_unset (&text);
+	g_value_unset (&visible);
+}
+
+void
+ephy_location_entry_set_completion_func (EphyLocationEntry *le, 
+					GtkEntryCompletionMatchFunc completion_func,
+					gpointer user_data)
+{
+	EphyLocationEntryPrivate *priv = le->priv;
+	GtkEntryCompletion *completion;
+	
+	completion = gtk_entry_get_completion (GTK_ENTRY (priv->icon_entry->entry));
+	gtk_entry_completion_set_match_func (completion, completion_func, user_data, NULL);
+}
+
 void
 ephy_location_entry_set_completion (EphyLocationEntry *le,
 				    GtkTreeModel *model,
@@ -1011,7 +1028,7 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 	EphyLocationEntryPrivate *priv = le->priv;
 	GtkTreeModel *sort_model;
 	GtkEntryCompletion *completion;
-	GtkCellRenderer *cell, *extracell, *iconcell;
+	GtkCellRenderer *cell, *iconcell;
 
 	le->priv->text_col = text_col;
 	le->priv->action_col = action_col;
@@ -1030,7 +1047,6 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 	completion = gtk_entry_completion_new ();
 	gtk_entry_completion_set_model (completion, sort_model);
 	g_object_unref (sort_model);
-	gtk_entry_completion_set_match_func (completion, completion_func, le, NULL);
 	g_signal_connect (completion, "match-selected",
 			  G_CALLBACK (match_selected_cb), le);
 	g_signal_connect_after (completion, "action-activated",
@@ -1053,19 +1069,20 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
 				       cell, "text", text_col);
 
-	extracell = gtk_cell_renderer_text_new ();
-	g_object_set (extracell, 
+	le->priv->extracell = gtk_cell_renderer_text_new ();
+	g_object_set (le->priv->extracell, 
 			"ellipsize", PANGO_ELLIPSIZE_END,
 			"ellipsize-set", TRUE,
-			"foreground", "Gray",
-			"foreground-set", TRUE,
-			"alignment", PANGO_ALIGN_RIGHT,
+			"alignment", PANGO_ALIGN_LEFT,
 			NULL);
 
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
-				    extracell, TRUE);
-	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
-				       extracell, "text", extra_col);
+				    le->priv->extracell, TRUE);
+	
+	gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (completion),
+					le->priv->extracell, extracell_data_func, 
+					GUINT_TO_POINTER (extra_col),
+					NULL);
 
 	g_object_set (completion, "inline-selection", TRUE, NULL);
 	g_signal_connect (completion, "cursor-on-match",
@@ -1088,7 +1105,7 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 
 	g_return_if_fail (address != NULL);
 
-        /* Setting a new text will clear the clipboard. This makes it impossible
+	/* Setting a new text will clear the clipboard. This makes it impossible
 	 * to copy&paste from the location entry of one tab into another tab, see
 	 * bug #155824. So we save the selection iff the clipboard was owned by
 	 * the location entry.
@@ -1126,9 +1143,9 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	/* First record the new hash, then update the entry text */
 	priv->hash = g_str_hash (address);
 
-	priv->user_changed = FALSE;
+	priv->block_update = TRUE;
 	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), text);
-	priv->user_changed = TRUE;
+	priv->block_update = FALSE;
 
 	/* We need to call update_address_state() here, as the 'changed' signal
 	 * may not get called if the user has typed in the exact correct url */
@@ -1146,6 +1163,22 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	}
 }
 
+gboolean
+ephy_location_entry_get_can_undo (EphyLocationEntry *entry)
+{
+	EphyLocationEntryPrivate *priv = entry->priv;
+	
+	return priv->user_changed;
+}
+
+gboolean
+ephy_location_entry_get_can_redo (EphyLocationEntry *entry)
+{
+	EphyLocationEntryPrivate *priv = entry->priv;
+	
+	return priv->can_redo;
+}
+
 const char *
 ephy_location_entry_get_location (EphyLocationEntry *entry)
 {
@@ -1168,6 +1201,10 @@ ephy_location_entry_reset_internal (EphyLocationEntry *entry,
 	old_text = gtk_entry_get_text (GTK_ENTRY (priv->icon_entry->entry));
 	old_text = old_text != NULL ? old_text : "";
 
+	g_free (priv->saved_text);
+	priv->saved_text = g_strdup (old_text);
+	priv->can_redo = TRUE;
+
 	retval = g_str_hash (text) != g_str_hash (old_text);
 
 	ephy_location_entry_set_location (entry, text, NULL);
@@ -1177,10 +1214,22 @@ ephy_location_entry_reset_internal (EphyLocationEntry *entry,
 	{
 		g_signal_emit (entry, signals[USER_CHANGED], 0);
 	}
+	
+	priv->user_changed = FALSE;
 
 	return retval;
 }
 
+void
+ephy_location_entry_undo_reset (EphyLocationEntry *entry)
+{
+	EphyLocationEntryPrivate *priv = entry->priv;
+	
+	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), priv->saved_text);
+	priv->can_redo = FALSE;
+	priv->user_changed = TRUE;
+}
+
 gboolean
 ephy_location_entry_reset (EphyLocationEntry *entry)
 {
@@ -1197,8 +1246,8 @@ ephy_location_entry_activate (EphyLocationEntry *entry)
 
 	gtk_editable_select_region (GTK_EDITABLE(priv->icon_entry->entry),
 				    0, -1);
-        gtk_window_set_focus (GTK_WINDOW(toplevel),
-                              priv->icon_entry->entry);
+	gtk_window_set_focus (GTK_WINDOW(toplevel),
+			      priv->icon_entry->entry);
 }
 
 GtkWidget *
