@@ -13,15 +13,15 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-search-entry.h 6588 2006-09-13 11:34:25Z chpe $
+ *  $Id: ephy-search-entry.h 7358 2007-09-09 17:49:34Z cosimoc $
  */
 
 #ifndef EPHY_SEARCH_ENTRY_H
 #define EPHY_SEARCH_ENTRY_H
 
-#include <gtk/gtkentry.h>
+#include "ephy-icon-entry.h"
 
 G_BEGIN_DECLS
 
@@ -38,7 +38,7 @@ typedef struct _EphySearchEntryClass	EphySearchEntryClass;
 
 struct _EphySearchEntry
 {
-	GtkEntry parent;
+	EphyIconEntry parent;
 
 	/*< private >*/
 	EphySearchEntryPrivate *priv;
@@ -46,7 +46,7 @@ struct _EphySearchEntry
 
 struct _EphySearchEntryClass
 {
-	GtkEntryClass parent;
+	EphyIconEntryClass parent;
 
 	void (*search) (EphySearchEntry *view, const char *text);
 };
